@@ -26,22 +26,21 @@
 #pod General topics like getting source, building, installing, bug reporting and some others are covered
 #pod in the F<README>.
 #pod
-#pod =head1 SYNOPSIS
-#pod
 #pod =for test_synopsis my $self;
 #pod
+#pod =head1 SYNOPSIS
+#pod
 #pod In your plugin:
 #pod
-#pod     # Iterate through the distribution files listed in MANIFEST
-#pod     # (files not included into distrubution are not iterated):
-#pod     my $files = $self->zilla->plugin_named( 'Manifest::Read' )->find_files();
-#pod     for my $file ( @$files ) {
+#pod     # Iterate through the distribution files listed in MANIFEST:
+#pod     my $finder = $self->zilla->plugin_named( 'Manifest::Read/:AllFiles' );
+#pod     for my $file ( @{ $finder->find_files() } ) {
 #pod         ...
 #pod     };
 #pod
 #pod =head1 DESCRIPTION
 #pod
-#pod This class consumes L<Dist::Zilla::Role::FileGatherer> and C<Dist::Zilla::Role::FileFinder> role.
+#pod This class consumes L<Dist::Zilla::Role::FileGatherer> and C<Dist::Zilla::Role::FileFinder> roles.
 #pod In order to fulfill requirements, the class implements C<gather_files> and C<find_files> methods.
 #pod Other methods are supporting.
 #pod
@@ -59,7 +58,7 @@ use namespace::autoclean;
 use version 0.77;
 
 # ABSTRACT: Read annotated source manifest
-our $VERSION = 'v0.4.2_01'; # TRIAL VERSION
+our $VERSION = 'v0.4.2_02'; # TRIAL VERSION
 
 with 'Dist::Zilla::Role::FileGatherer';
 with 'Dist::Zilla::Role::FileFinder';
@@ -125,7 +124,7 @@ sub _build_manifest_file {
 
 #pod =method BUILD
 #pod
-#pod This method creates bunch of file finders: C<Manifest::Write/AllFiles>, C<Manifest::Write/ExecFiles>, C<Manifest::Write/ExtraTestFiles>, C<Manifest::Write/InstallModules>, C<Manifest::Write/PerlExecFiles>, C<Manifest::Write/TestFiles>.
+#pod This method creates bunch of file finders: C<Manifest::Read/:AllFiles>, C<Manifest::Read/:ExecFiles>, C<Manifest::Read/:ExtraTestFiles>, C<Manifest::Read/:InstallModules>, C<Manifest::Read/:PerlExecFiles>, C<Manifest::Read/:TestFiles>.
 #pod
 #pod =cut
 
@@ -134,23 +133,27 @@ sub BUILD {
     require Dist::Zilla::Plugin::FinderCode;
     my $parent = $self;
     my $zilla  = $self->zilla;
-    my @finders = qw{ AllFiles ExecFiles ExtraTestFiles InstallModules PerlExecFiles TestFiles };
+    my @finders = qw{ :AllFiles :ExecFiles :ExtraTestFiles :InstallModules :PerlExecFiles :TestFiles };
     for my $name ( @finders ) {
         my $finder = Dist::Zilla::Plugin::FinderCode->new( {
-            plugin_name => $self->plugin_name . '/' . $name,
+            plugin_name => $parent->plugin_name . '/' . $name,
             zilla       => $zilla,
             style       => 'list',
             code        => sub {
                 my ( $self ) = @_;
-                my $plugin = $self->zilla->plugin_named( ':' . $name );
+                my $plugin = $self->zilla->plugin_named( $name );
                 if ( not defined( $plugin ) ) {
-                    $parent->abort( [ "Can't find plugin %s", ':' . $name ] );
+                    $parent->abort( [ "Can't find plugin %s", $name ] );
                 };
-                my @files = grep(
-                    { ( $parent->_file_hash->{ $_->name } || 0 ) == $_ }
-                    @{ $plugin->find_files() }
-                );
-                return \@files;
+                my $m = $parent->_manifest_bulk;
+                my $files = [];
+                for my $file ( @{ $plugin->find_files() } ) {
+                    my $item = $m->{ $file->name } or next;
+                    $item->{ file } == $file       or next;
+                    $item->{ mark } ne '-'         or next;
+                    push( @$files, $file );
+                };
+                return $files;
             },
         } );
         push( @{ $zilla->plugins }, $finder );
@@ -163,14 +166,13 @@ sub BUILD {
 #pod =method gather_files
 #pod
 #pod This method fulfills L<Dist::Zilla::Role::FileGatherer> role requirement. It adds files listed in
-#pod manifest to distribution (and to C<_file_list>). Files marked to exclude from distribution and
-#pod directories are not added.
+#pod manifest to distribution. Files marked to exclude from distribution and directories are not added.
 #pod
 #pod =cut
 
 sub gather_files {
     my ( $self ) = @_;
-    for my $file ( @{ $self->_file_list } ) {
+    for my $file ( @{ $self->_incl_file_list } ) {
         $self->add_file( $file );
     };
     return;
@@ -180,102 +182,111 @@ sub gather_files {
 
 #pod =method find_files
 #pod
-#pod This method fulfills L<Dist::Zilla::Role::FileFinder> role requirement. It simply returns a copy of
-#pod C<_file_list> attribute.
+#pod This method fulfills L<Dist::Zilla::Role::FileFinder> role requirement. It returns the I<complete>
+#pod list (strictly speaking, arrayref) of files read from the manifest, in order of appearance.
 #pod
-#pod This method can be called by other plugins to iterate through files added by C<Manifest::Read>,
-#pod see L</"SYNOPSIS">.
+#pod Note: The list includes files which are I<not> added to the distribution.
 #pod
 #pod Note: The method always returns the same list of files. Plugins which remove files from
 #pod distribution (i. e. plugins which do C<Dist::Zilla::Role::FilePruner> role) do not affect result of
 #pod the method.
 #pod
+#pod If you are interested in distribution files, have look to file finders generated by C<BUILD>.
+#pod
 #pod =cut
 
 sub find_files {
     my ( $self ) = @_;
-    return [ @{ $self->_file_list } ];
+    return [
+        map(
+            { $_->{ file } }
+            sort(
+                { $a->{ line } <=> $b->{ line } }
+                values( %{ $self->_manifest_bulk } )
+            )
+        )
+    ];
+    #~ return [ @{ $self->_incl_file_list } ];
 };
 
 # --------------------------------------------------------------------------------------------------
 
-#pod =attr _file_list
+#pod =attr _manifest_bulk
 #pod
-#pod Array of files (object which do C<Dist::Zilla::Role::File> role) listed in the manifest I<and>
-#pod marked for inclusion to the distribution.
+#pod Parsed manifest. Hash of hashes. Each inner hash has keys and associated values:
 #pod
-#pod C<ArrayRef>, read-only, lazy, initialized with builder.
+#pod =for :list
+#pod =   name
+#pod Parsed filename (single-quoted filenames are unquoted, escape sequences are evaluated, if any).
+#pod =   file
+#pod Object which does C<Dist::Zilla::Role::File> role.
+#pod =   mark
+#pod Mark.
+#pod =   comment
+#pod File comment, leading and trailing whitespaces are stripped.
+#pod =   line
+#pod Number of manifest line the file listed in.
+#pod
+#pod C<HasfRef>, read-only, lazy, initialized with builder.
 #pod
 #pod =cut
 
-has _file_list => (
-    isa         => 'ArrayRef[Object]',
+has _manifest_bulk => (
+    isa         => 'HashRef[HashRef]',
     is          => 'ro',
     lazy        => 1,
-    builder     => '_build_file_list',
+    builder     => '_build_manifest_bulk',
     init_arg    => undef,
 );
 
-sub _build_file_list {
+sub _build_manifest_bulk {
     my ( $self ) = @_;
-    my $files = [];
+    my $items = {};
     my @errors;
     my $error = sub {
         my ( $item, $message ) = @_;
         my $err = sprintf(
             '%s %s at %s line %d.',
-            $item->{ filename }, $message, $self->manifest_name, $item->{ line },
+            $item->{ name }, $message, $self->manifest_name, $item->{ line },
         );
         push( @errors, $item->{ line } => $err );
         return $self->log_error( $err );
     };
     foreach my $item ( $self->_parse_lines() ) {
-        -e $item->{ filename } or $error->( $item, 'does not exist' ) and next;
+        -e $item->{ name } or $error->( $item, 'does not exist' ) and next;
         if ( $item->{ mark } eq '/' ) {
             -d _ or $error->( $item, 'is not a directory' ) and next;
         } else {
             -f _ or $error->( $item, 'is not a plain file' ) and next;
-            if ( $item->{ mark } ne '-' ) {
-                my $file = Dist::Zilla::File::OnDisk->new( { name => $item->{ filename } } );
-                push( @$files, $file );
-            };
+            $item->{ file } = Dist::Zilla::File::OnDisk->new( { name => $item->{ name } } );
+            $items->{ $item->{ name } } = $item;
         };
     };
     if ( @errors ) {
         $self->log_errors_in_file( $self->manifest_file, @errors );
     };
     $self->abort_if_error();
-    return $files;
+    return $items;
 };
 
 # --------------------------------------------------------------------------------------------------
 
-#pod =attr _file_hash
+#pod =method _incl_file_list
 #pod
-#pod Hash of files (object which do C<Dist::Zilla::Role::File> role) listed in the manifest I<and>
-#pod marked for inclusion to the distribution. Hash keys are file names, hash values are references to
-#pod file objects. Built from C<_file_list>.
-#pod
-#pod C<HasfRef>, read-only, lazy, initialized with builder.
+#pod Returns array of files (object which do C<Dist::Zilla::Role::File> role) listed in the manifest
+#pod I<and> marked for inclusion to the distribution.
 #pod
 #pod =cut
 
-has _file_hash => (
-    isa         => 'HashRef[Object]',
-    is          => 'ro',
-    lazy        => 1,
-    builder     => '_build_file_hash',
-    init_arg    => undef,
-);
-
-sub _build_file_hash {
+sub _incl_file_list {
     my ( $self ) = @_;
-    return { map( { $_->name => $_ } @{ $self->_file_list } ) };
+    my $m = $self->_manifest_bulk;
+    return [ map( { $m->{ $_ }->{ file } } grep( { $m->{ $_ }->{ mark } ne '-' } keys( %$m ) ) ) ];
 };
 
 # --------------------------------------------------------------------------------------------------
 
-#pod =attr _lines
+#pod =attr _manifest_lines
 #pod
 #pod Array of chomped manifest lines, including comments and empty lines.
 #pod
@@ -283,15 +294,15 @@ sub _build_file_hash {
 #pod
 #pod =cut
 
-has _lines => (
+has _manifest_lines => (
     isa         => 'ArrayRef[Str]',
     is          => 'ro',
     lazy        => 1,
     init_arg    => undef,
-    builder     => '_build_lines',
+    builder     => '_build_manifest_lines',
 );
 
-sub _build_lines {
+sub _build_manifest_lines {
     my ( $self ) = @_;
     my $lines = [];
     try {
@@ -321,22 +332,22 @@ sub _build_lines {
 #pod The method returns list of hashrefs, a hash per file. Each hash has following keys and values:
 #pod
 #pod =for :list
-#pod =   filename
+#pod =   name
 #pod Parsed filename (single-quoted filenames are unquoted, escape sequences are evaluated, if any).
 #pod =   mark
 #pod Mark.
 #pod =   comment
 #pod File comment, leading and trailing whitespaces are stripped.
 #pod =   line
-#pod Number of manifest line the file listed in.
+#pod Number of manifest line (one-based) the file is listed in.
 #pod
 #pod =cut
 
 my %RE = (
-    filename => qr{ ' (*PRUNE) (?: [^'\\] ++ | \\ ['\\] ?+ ) ++ ' | \S ++ }x,
+    name    => qr{ ' (*PRUNE) (?: [^'\\] ++ | \\ ['\\] ?+ ) ++ ' | \S ++ }x,
         # ^ TODO: Use Regexp::Common for quoted filename?
-    mark     => qr{ [#/+-] }x,
-    comment  => qr{ . *? }x,
+    mark    => qr{ [#/+-] }x,
+    comment => qr{ . *? }x,
 );
 
 sub _parse_lines {
@@ -345,7 +356,7 @@ sub _parse_lines {
     my ( %files, @files );
     my @errors;
     my $n = 0;
-    for my $line ( @{ $self->_lines } ) {
+    for my $line ( @{ $self->_manifest_lines } ) {
         ++ $n;
         if ( $line =~ m{ \A \s * (?: \# | \z ) }x ) {   # Comment or empty line.
             next;
@@ -354,7 +365,7 @@ sub _parse_lines {
         $line =~ m{
             \A
             \s *+                           # requires perl v5.10
-            ( $RE{ filename } )
+            ( $RE{ name } )
             (*PRUNE)                        # requires perl v5.10
             (?:
                 \s ++
@@ -368,13 +379,13 @@ sub _parse_lines {
             \s *
             \z
         }x and do {
-            my ( $filename, $mark, $comment ) = ( $1, $2, $3 );
-            if ( $filename =~ s{ \A ' ( . * ) ' \z }{ $1 }ex ) {
-                $filename =~ s{  \\ ( ['\\] ) }{ $1 }gex;
+            my ( $name, $mark, $comment ) = ( $1, $2, $3 );
+            if ( $name =~ s{ \A ' ( . * ) ' \z }{ $1 }ex ) {
+                $name =~ s{  \\ ( ['\\] ) }{ $1 }gex;
             };
-            if ( exists( $files{ $filename } ) ) {
-                my $f = $files{ $filename };
-                $self->log_error( [ '%s at %s line %d', $filename, $manifest, $n ] );
+            if ( exists( $files{ $name } ) ) {
+                my $f = $files{ $name };
+                $self->log_error( [ '%s at %s line %d', $name, $manifest, $n ] );
                 $self->log_error( [ '    also listed at %s line %d.', $manifest, $f->{ line } ] );
                 push( @errors,
                     $n           => 'The file also listed at line ' . $f->{ line } . '.',
@@ -383,12 +394,12 @@ sub _parse_lines {
                 next;
             };
             my $file = {
-                filename => $filename,
-                mark     => $mark  // '+',     # requires perl v5.10
-                comment  => $comment,
-                line     => $n,
+                name    => $name,
+                mark    => $mark  // '+',     # requires perl v5.10
+                comment => $comment,
+                line    => $n,
             };
-            $files{ $filename } = $file;
+            $files{ $name } = $file;
             push( @files, $file );
             1;
         } or do {
@@ -422,7 +433,7 @@ __PACKAGE__->meta->make_immutable;
 #pod C<Dist-Zilla-Plugin-Manifest-Read> (or C<Manifest::Read> for brevity) is a C<Dist::Zilla> plugin. It reads
 #pod I<annotated source> manifest, checks existence of all listed files and directories, and adds
 #pod selected files to the distribution. C<Manifest::Read> also does C<FileFinder> role, providing the
-#pod list of added files for other plugins.
+#pod list of files for other plugins.
 #pod
 #pod =cut
 
@@ -461,7 +472,7 @@ Dist::Zilla::Plugin::Manifest::Read - Read annotated source manifest
 
 =head1 VERSION
 
-Version v0.4.2_01, released on 2016-10-11 00:15 UTC.
+Version v0.4.2_02, released on 2016-11-04 23:54 UTC.
 This is a B<trial release>.
 
 =head1 WHAT?
@@ -469,7 +480,7 @@ This is a B<trial release>.
 C<Dist-Zilla-Plugin-Manifest-Read> (or C<Manifest::Read> for brevity) is a C<Dist::Zilla> plugin. It reads
 I<annotated source> manifest, checks existence of all listed files and directories, and adds
 selected files to the distribution. C<Manifest::Read> also does C<FileFinder> role, providing the
-list of added files for other plugins.
+list of files for other plugins.
 
 This is C<Dist::Zilla::Plugin::Manifest::Read> module documentation. Read this if you are going to hack or
 extend C<Manifest::Read>, or use it programmatically.
@@ -478,11 +489,21 @@ If you want to have annotated source manifest, read the L<user manual|Dist::Zill
 General topics like getting source, building, installing, bug reporting and some others are covered
 in the F<README>.
 
+=for test_synopsis my $self;
+
 =head1 SYNOPSIS
 
+In your plugin:
+
+    # Iterate through the distribution files listed in MANIFEST:
+    my $finder = $self->zilla->plugin_named( 'Manifest::Read/:AllFiles' );
+    for my $file ( @{ $finder->find_files() } ) {
+        ...
+    };
+
 =head1 DESCRIPTION
 
-This class consumes L<Dist::Zilla::Role::FileGatherer> and C<Dist::Zilla::Role::FileFinder> role.
+This class consumes L<Dist::Zilla::Role::FileGatherer> and C<Dist::Zilla::Role::FileFinder> roles.
 In order to fulfill requirements, the class implements C<gather_files> and C<find_files> methods.
 Other methods are supporting.
 
@@ -503,67 +524,20 @@ Manifest file as a C<Dist::Zilla> file object (C<Dist::Zilla::File::OnDisk>).
 
 C<Object>, read-only.
 
-=head2 _file_list
+=head2 _manifest_bulk
 
-Array of files (object which do C<Dist::Zilla::Role::File> role) listed in the manifest I<and>
-marked for inclusion to the distribution.
-
-C<ArrayRef>, read-only, lazy, initialized with builder.
-
-=head2 _file_hash
-
-Hash of files (object which do C<Dist::Zilla::Role::File> role) listed in the manifest I<and>
-marked for inclusion to the distribution. Hash keys are file names, hash values are references to
-file objects. Built from C<_file_list>.
-
-C<HasfRef>, read-only, lazy, initialized with builder.
-
-=head2 _lines
-
-Array of chomped manifest lines, including comments and empty lines.
-
-C<ArrayRef[Str]>, read-only, lazy, initialized with builder.
-
-=head1 OBJECT METHODS
-
-=head2 BUILD
-
-This method creates bunch of file finders: C<Manifest::Write/AllFiles>, C<Manifest::Write/ExecFiles>, C<Manifest::Write/ExtraTestFiles>, C<Manifest::Write/InstallModules>, C<Manifest::Write/PerlExecFiles>, C<Manifest::Write/TestFiles>.
-
-=head2 gather_files
-
-This method fulfills L<Dist::Zilla::Role::FileGatherer> role requirement. It adds files listed in
-manifest to distribution (and to C<_file_list>). Files marked to exclude from distribution and
-directories are not added.
-
-=head2 find_files
-
-This method fulfills L<Dist::Zilla::Role::FileFinder> role requirement. It simply returns a copy of
-C<_file_list> attribute.
-
-This method can be called by other plugins to iterate through files added by C<Manifest::Read>,
-see L</"SYNOPSIS">.
-
-Note: The method always returns the same list of files. Plugins which remove files from
-distribution (i. e. plugins which do C<Dist::Zilla::Role::FilePruner> role) do not affect result of
-the method.
-
-=head2 _parse_lines
-
-This method parses manifest lines. Each line is parsed separately (there is no line continuation).
-
-If the method fails to parse a line, error is reported by calling method C<log_error> (implemented
-in L<Dist::Zilla::Role::ErrorLogger>). This means that parsing is not stopped at the first failure,
-but entire manifest will be parsed and all the found errors will be reported.
-
-The method returns list of hashrefs, a hash per file. Each hash has following keys and values:
+Parsed manifest. Hash of hashes. Each inner hash has keys and associated values:
 
 =over 4
 
-=item filename
+=item name
 
 Parsed filename (single-quoted filenames are unquoted, escape sequences are evaluated, if any).
 
+=item file
+
+Object which does C<Dist::Zilla::Role::File> role.
+
 =item mark
 
 Mark.
@@ -578,16 +552,72 @@ Number of manifest line the file listed in.
 
 =back
 
-=for test_synopsis my $self;
+C<HasfRef>, read-only, lazy, initialized with builder.
 
-In your plugin:
+=head2 _manifest_lines
 
-    # Iterate through the distribution files listed in MANIFEST
-    # (files not included into distrubution are not iterated):
-    my $files = $self->zilla->plugin_named( 'Manifest::Read' )->find_files();
-    for my $file ( @$files ) {
-        ...
-    };
+Array of chomped manifest lines, including comments and empty lines.
+
+C<ArrayRef[Str]>, read-only, lazy, initialized with builder.
+
+=head1 OBJECT METHODS
+
+=head2 BUILD
+
+This method creates bunch of file finders: C<Manifest::Read/:AllFiles>, C<Manifest::Read/:ExecFiles>, C<Manifest::Read/:ExtraTestFiles>, C<Manifest::Read/:InstallModules>, C<Manifest::Read/:PerlExecFiles>, C<Manifest::Read/:TestFiles>.
+
+=head2 gather_files
+
+This method fulfills L<Dist::Zilla::Role::FileGatherer> role requirement. It adds files listed in
+manifest to distribution. Files marked to exclude from distribution and directories are not added.
+
+=head2 find_files
+
+This method fulfills L<Dist::Zilla::Role::FileFinder> role requirement. It returns the I<complete>
+list (strictly speaking, arrayref) of files read from the manifest, in order of appearance.
+
+Note: The list includes files which are I<not> added to the distribution.
+
+Note: The method always returns the same list of files. Plugins which remove files from
+distribution (i. e. plugins which do C<Dist::Zilla::Role::FilePruner> role) do not affect result of
+the method.
+
+If you are interested in distribution files, have look to file finders generated by C<BUILD>.
+
+=head2 _incl_file_list
+
+Returns array of files (object which do C<Dist::Zilla::Role::File> role) listed in the manifest
+I<and> marked for inclusion to the distribution.
+
+=head2 _parse_lines
+
+This method parses manifest lines. Each line is parsed separately (there is no line continuation).
+
+If the method fails to parse a line, error is reported by calling method C<log_error> (implemented
+in L<Dist::Zilla::Role::ErrorLogger>). This means that parsing is not stopped at the first failure,
+but entire manifest will be parsed and all the found errors will be reported.
+
+The method returns list of hashrefs, a hash per file. Each hash has following keys and values:
+
+=over 4
+
+=item name
+
+Parsed filename (single-quoted filenames are unquoted, escape sequences are evaluated, if any).
+
+=item mark
+
+Mark.
+
+=item comment
+
+File comment, leading and trailing whitespaces are stripped.
+
+=item line
+
+Number of manifest line (one-based) the file is listed in.
+
+=back
 
 =head1 SEE ALSO
 
